@@ -68,7 +68,7 @@ import types
 import weakref
 
 import classregistry
-from converters import sqlrepr, registerConverter
+from converters import registerConverter, sqlrepr, quote_str, unquote_str
 
 
 class VersionError(Exception):
@@ -95,6 +95,12 @@ def execute(expr, executor):
     else:
         return expr
 
+
+def _str_or_sqlrepr(expr, db):
+    if isinstance(expr, basestring):
+        return expr
+    return sqlrepr(expr, db)
+
 ########################################
 ## Expression generation
 ########################################
@@ -127,7 +133,7 @@ class SQLExpression:
     def __abs__(self):
         return SQLConstant("ABS")(self)
     def __mod__(self, other):
-        return SQLModulo(self, other)
+        return SQLConstant("MOD")(self, other)
     def __rmod__(self, other):
         return SQLConstant("MOD")(other, self)
 
@@ -259,18 +265,7 @@ class SQLOp(SQLExpression):
             return operatorMap[self.op.upper()](execute(self.expr1, executor),
                                                 execute(self.expr2, executor))
 
-class SQLModulo(SQLOp):
-    def __init__(self, expr1, expr2):
-        SQLOp.__init__(self, '%', expr1, expr2)
-    def __sqlrepr__(self, db):
-        if db == 'sqlite':
-            return SQLOp.__sqlrepr__(self, db)
-        s1 = sqlrepr(self.expr1, db)
-        s2 = sqlrepr(self.expr2, db)
-        return "MOD(%s, %s)" % (s1, s2)
-
 registerConverter(SQLOp, SQLExprConverter)
-registerConverter(SQLModulo, SQLExprConverter)
 
 class SQLCall(SQLExpression):
     def __init__(self, expr, args):
@@ -383,9 +378,7 @@ class Table(SQLExpression):
             raise AttributeError
         return self.FieldClass(self.tableName, attr)
     def __sqlrepr__(self, db):
-        if isinstance(self.tableName, str):
-            return self.tableName
-        return sqlrepr(self.tableName, db)
+        return _str_or_sqlrepr(self.tableName, db)
     def execute(self, executor):
         raise ValueError, "Tables don't have values"
 
@@ -627,11 +620,11 @@ class Select(SQLExpression):
         if self.ops['distinct']:
             select += " DISTINCT"
             if self.ops['distinctOn'] is not NoDefault:
-                select += " ON(%s)" % sqlrepr(self.ops['distinctOn'], db)
+                select += " ON(%s)" % _str_or_sqlrepr(self.ops['distinctOn'], db)
         if not self.ops['lazyColumns']:
-            select += " %s" % ", ".join([str(sqlrepr(v, db)) for v in self.ops['items']])
+            select += " %s" % ", ".join([str(_str_or_sqlrepr(v, db)) for v in self.ops['items']])
         else:
-            select += " %s" % sqlrepr(self.ops['items'][0], db)
+            select += " %s" % _str_or_sqlrepr(self.ops['items'][0], db)
 
         join = []
         join_str = ''
@@ -655,8 +648,9 @@ class Select(SQLExpression):
             if isinstance(thing, SQLExpression):
                 tables.update(tablesUsedSet(thing, db))
         for j in join:
-            t1, t2 = sqlrepr(j.table1, db), sqlrepr(j.table2, db)
+            t1 = _str_or_sqlrepr(j.table1, db)
             if t1 in tables: tables.remove(t1)
+            t2 = _str_or_sqlrepr(j.table2, db)
             if t2 in tables: tables.remove(t2)
         if tables:
             select += " FROM %s" % ", ".join(tables)
@@ -675,14 +669,14 @@ class Select(SQLExpression):
             select += join_str
 
         if self.ops['clause'] is not NoDefault:
-            select += " WHERE %s" % sqlrepr(self.ops['clause'], db)
+            select += " WHERE %s" % _str_or_sqlrepr(self.ops['clause'], db)
         if self.ops['groupBy'] is not NoDefault:
-            groupBy = sqlrepr(self.ops['groupBy'], db)
+            groupBy = _str_or_sqlrepr(self.ops['groupBy'], db)
             if isinstance(self.ops['groupBy'], (list, tuple)):
                 groupBy = groupBy[1:-1] # Remove parens
             select += " GROUP BY %s" % groupBy
         if self.ops['having'] is not NoDefault:
-            select += " HAVING %s" % sqlrepr(self.ops['having'], db)
+            select += " HAVING %s" % _str_or_sqlrepr(self.ops['having'], db)
         if self.ops['orderBy'] is not NoDefault and self.ops['orderBy'] is not None:
             orderBy = self.ops['orderBy']
             if self.ops['reversed']:
@@ -690,9 +684,9 @@ class Select(SQLExpression):
             else:
                 reverser = lambda x: x
             if isinstance(orderBy, (list, tuple)):
-                select += " ORDER BY %s" % ", ".join([sqlrepr(reverser(x), db) for x in orderBy])
+                select += " ORDER BY %s" % ", ".join([_str_or_sqlrepr(reverser(x), db) for x in orderBy])
             else:
-                select += " ORDER BY %s" % sqlrepr(reverser(orderBy), db)
+                select += " ORDER BY %s" % _str_or_sqlrepr(reverser(orderBy), db)
         start, end = self.ops['start'], self.ops['end']
         if self.ops['limit'] is not NoDefault:
             end = start + self.ops['limit']
@@ -775,7 +769,7 @@ class Update(SQLExpression):
                     update += ","
                 update += " %s=%s" % (key, sqlrepr(value, db))
         if self.whereClause is not NoDefault:
-            update += " WHERE %s" % sqlrepr(self.whereClause, db)
+            update += " WHERE %s" % _str_or_sqlrepr(self.whereClause, db)
         return update
     def sqlName(self):
         return "UPDATE"
@@ -794,8 +788,7 @@ class Delete(SQLExpression):
         whereClause = self.whereClause
         if whereClause is None:
             return "DELETE FROM %s" % self.table
-        if isinstance(whereClause, SQLExpression):
-            whereClause = sqlrepr(whereClause, db)
+        whereClause = _str_or_sqlrepr(whereClause, db)
         return "DELETE FROM %s WHERE %s" % (self.table, whereClause)
 
 registerConverter(Delete, SQLExprConverter)
@@ -911,18 +904,18 @@ class _LikeQuoted:
         if isinstance(s, SQLExpression):
             values = []
             if self.prefix:
-                values.append("'%s'" % self.prefix)
+                values.append(quote_str(self.prefix, db))
             s = _quote_like_special(sqlrepr(s, db), db)
             values.append(s)
             if self.postfix:
-                values.append("'%s'" % self.postfix)
+                values.append(quote_str(self.postfix, db))
             if db == "mysql":
                 return "CONCAT(%s)" % ", ".join(values)
             else:
                 return " || ".join(values)
         elif isinstance(s, basestring):
-            s = _quote_like_special(sqlrepr(s, db)[1:-1], db)
-            return "'%s%s%s'" % (self.prefix, s, self.postfix)
+            s = _quote_like_special(unquote_str(sqlrepr(s, db)), db)
+            return quote_str("%s%s%s" % (self.prefix, s, self.postfix), db)
         else:
            raise TypeError, "expected str, unicode or SQLExpression, got %s" % type(s)
 
